@@ -1,12 +1,11 @@
 """Implements the "Bottle" interface."""
-import uuid
-from typing import Dict, List, Any
 import io
 import json
+import uuid
+from typing import Dict, List, Any, Tuple, Union
 
 import requests
 
-from ialib.genie_metalanguage import *
 from ialib.genome_info import Genome
 
 
@@ -19,6 +18,7 @@ class ConnectionError(Exception):
     """Raised if any query to any node returns an error."""
     pass
 
+
 def remove_unique_id(response: dict) -> dict:
     """Return *response* with the key 'unique_id' removed regardless of nesting."""
     if isinstance(response, dict):
@@ -26,12 +26,13 @@ def remove_unique_id(response: dict) -> dict:
             del (response['unique_id'])
         for value in response.values():
             if isinstance(value, dict):
-                 remove_unique_id(value)
+                remove_unique_id(value)
     return response
 
 
 class BottleClient:
     """Interface for interacting with bottles."""
+
     def __init__(self, bottle_info):
         """
         Provide bottle information in a dictionary.
@@ -76,8 +77,9 @@ class BottleClient:
     def __repr__(self) -> str:
         return '<{name}.{domain}| secure: %r, connected: %s, genie: %s, \
                   ingress_nodes: %i, query_nodes: %i, failures: %i>'.format(
-                      **self.bottle_info) % (
-                          self.secure, self._connected, self.genie, len(self.ingress_nodes), len(self.query_nodes), len(self.failures))
+            **self.bottle_info) % (
+                   self.secure, self._connected, self.genie, len(self.ingress_nodes), len(self.query_nodes),
+                   len(self.failures))
 
     def receive_unique_ids(self, should_set: bool = True) -> bool:
         self.send_unique_ids = should_set
@@ -114,14 +116,16 @@ class BottleClient:
         self.query_nodes = [{'id': self.genome.primitive_map[node], 'name': node} for node in nodes]
         return self.query_nodes
 
-    def _query(self, query_method: Any, path: str, data: Dict = None, nodes: List = None, unique_id: str = None) -> List:
+    def _query(self, query_method: Any, path: str, data: Dict = None, nodes: List = None,
+               unique_id: str = None) -> Union[dict, Tuple[dict, str]]:
         """Internal helper function to make an RPC call with the given *query* and *data*."""
         if not self._connected:
-            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
+            raise ConnectionError(
+                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
         result = {}
         if unique_id is not None:
             if data:
-                    data['unique_id'] = unique_id
+                data['unique_id'] = unique_id
             else:
                 data = {'unique_id': unique_id}
 
@@ -138,7 +142,7 @@ class BottleClient:
                 response.raise_for_status()
                 response = response.json()
                 if response['status'] != 'okay':
-                    self.failures.append({node['name']: {'message': response['messsage']}})
+                    self.failures.append({node['name']: {'message': response['message']}})
                 if not self.send_unique_ids:
                     response = remove_unique_id(response['message'])
                 else:
@@ -149,7 +153,8 @@ class BottleClient:
                     result[node['name']] = response
             except Exception as exception:
                 self.failures.append({node['name']: {'class': exception.__class__.__name__, 'message': str(exception)}})
-                raise QueryError("Query Failure:", {'class': exception.__class__.__name__, 'message': str(exception)}) from exception
+                raise QueryError("Query Failure:",
+                                 {'class': exception.__class__.__name__, 'message': str(exception)}) from exception
         if unique_id is not None:
             return result, unique_id
         return result
@@ -157,24 +162,21 @@ class BottleClient:
     def set_summarize_for_single_node(self, value: bool):
         self.summarize_for_single_node = value
 
-    def observe(self, data: Dict = None, nodes: List = None) -> List:
+    def observe(self, data: Dict, nodes: List = None) -> Union[dict, Tuple[dict, str]]:
         """Exclusively uses the 'observe' call.  All commands must be provided via Genie Metalanguage data."""
         if nodes is None:
             nodes = self.ingress_nodes
         return self._query(requests.post, 'observe', data=data, nodes=nodes)
 
-    def observe_event(self, data: Dict = None, unique_id: str = None) -> List:
+    def observe_event(self, data: Dict, unique_id: str = None) -> Tuple[dict, str]:
         """Exclusively uses the 'observe' call.  All commands must be provided via Genie Metalanguage data."""
         results = {}
+        uid = None
         if unique_id is None:
             unique_id = str(uuid.uuid4())
         for node, node_data in data.items():
-            response, uid = self._query(requests.post, 'observe', node_data, nodes=[node], unique_id=unique_id)
-            results.append(result)
-            response['unique_id'] = uid
-            results[node['name']] = response
-        if len(results) == 1:
-            results = results[0]
+            response, uid = self._query(requests.post, 'observe', data=node_data, nodes=[node], unique_id=unique_id)
+            results[node] = response
         return results, uid
 
     def observe_classification(self, data=None, nodes: List = None):
@@ -183,7 +185,8 @@ class BottleClient:
         This function does that for us.
         """
         if not self._connected:
-            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
+            raise ConnectionError(
+                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
         if nodes is None:
             nodes = self.query_nodes
         return self._query("observe", data, nodes=nodes)
@@ -266,21 +269,22 @@ class BottleClient:
         Only works on primitive nodes at this time.
         """
         if not self._connected:
-            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
+            raise ConnectionError(
+                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
         if nodes is None:
             nodes = self.all_nodes
         else:
             nodes = [node for node in self.all_nodes if (node['name'] in nodes)]
-        
+
         result = {}
         for node in nodes:  ## only primitive nodes at this time.
-            response = requests.post(f"{self.url}{node['id']}/genes/change", 
-                                        verify=self.secure, headers=self.headers, 
-                                        json={'data': gene_data}).json()
+            response = requests.post(f"{self.url}{node['id']}/genes/change",
+                                     verify=self.secure, headers=self.headers,
+                                     json={'data': gene_data}).json()
             if 'error' in response or response['status'] == 'failed':
                 if len(nodes) == 1 and self.summarize_for_single_node:
                     raise Exception("Request Failure:", response)
-                self.failures.append({node['name']: exception})
+                self.failures.append({node['name']: response})
                 print("Failure:", {node['name']: response})
             self.genome.change_genes(node['id'], gene_data)
             if len(nodes) == 1 and self.summarize_for_single_node:
@@ -294,10 +298,12 @@ class BottleClient:
         *genome* must be either a JSON-serializable object or a file-like object.
         """
         if not self._connected:
-            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
+            raise ConnectionError(
+                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
         if isinstance(genome, io.TextIOBase):
             genome = json.load(genome)
-        response = requests.post(f'{self.url}genome/inject', verify=self.secure, headers=self.headers, json={'genome': genome}).json()
+        response = requests.post(f'{self.url}genome/inject', verify=self.secure, headers=self.headers,
+                                 json={'genome': genome}).json()
         if 'error' not in response:
             self.genome = Genome(genome)
             self.genie = self.genome.agent
@@ -323,7 +329,8 @@ class BottleClient:
         Only works on primitive nodes at this time.
         """
         if not self._connected:
-            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
+            raise ConnectionError(
+                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
         if nodes is None:
             nodes = self.all_nodes
         else:
@@ -331,16 +338,16 @@ class BottleClient:
         result = {}
         for node in nodes:
             try:
-                response = requests.get(f"{self.url}{node['id']}/gene/{gene}", 
+                response = requests.get(f"{self.url}{node['id']}/gene/{gene}",
                                         verify=self.secure,
                                         headers=self.headers).json()
                 if 'error' in response or response['status'] == 'failed':
                     if len(nodes) == 1 and self.summarize_for_single_node:
                         raise Exception("Request Failure:", response)
-                    self.failures.append({node['name']: exception})
+                    self.failures.append({node['name']: response})
                 if len(nodes) == 1 and self.summarize_for_single_node:
-                    return {gene : response['message']}
-                result[node['name']] = {gene : response['message']}
+                    return {gene: response['message']}
+                result[node['name']] = {gene: response['message']}
             except Exception as exception:
                 self.failures.append({node['name']: exception})
                 raise Exception("Request Failure:", {node['name']: exception})
@@ -358,7 +365,8 @@ class BottleClient:
         conveyed?
         """
         if not self._connected:
-            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
+            raise ConnectionError(
+                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
         if nodes is None:
             nodes = self.all_nodes
         else:
@@ -366,13 +374,13 @@ class BottleClient:
         result = {}
         for node in nodes:
             try:
-                response = requests.get(f"{self.url}{node['id']}/model/{model_name}", 
-                                        headers=self.headers, 
+                response = requests.get(f"{self.url}{node['id']}/model/{model_name}",
+                                        headers=self.headers,
                                         verify=self.secure).json()
                 if 'error' in response or response['status'] == 'failed':
                     if len(nodes) == 1 and self.summarize_for_single_node:
                         raise Exception("Request Failure:", response)
-                    self.failures.append({node['name']: exception})
+                    self.failures.append({node['name']: response})
                 if len(nodes) == 1 and self.summarize_for_single_node:
                     return response['message']
                 else:
@@ -401,7 +409,8 @@ class BottleClient:
         """
         """
         if not self._connected:
-            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
+            raise ConnectionError(
+                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
         if nodes is None:
             nodes = self.all_nodes
         else:
@@ -409,12 +418,16 @@ class BottleClient:
         result = {}
         for node in nodes:
             try:
-                response = requests.post(self.url, verify=self.secure, json={"method": "getVector", "params": {"api_key": self.api_key, "primitive_id": node['id'], "data": vector_name},
-                     "jsonrpc": "2.0", "id": 1}).json()['result']
+                response = requests.post(self.url, verify=self.secure, json={"method": "getVector",
+                                                                             "params": {"api_key": self.api_key,
+                                                                                        "primitive_id": node['id'],
+                                                                                        "data": vector_name},
+                                                                             "jsonrpc": "2.0", "id": 1}).json()[
+                    'result']
                 if 'error' in response or response['status'] == 'failed':
                     if len(nodes) == 1 and self.summarize_for_single_node:
                         raise Exception("Request Failure:", response)
-                    self.failures.append({node['name']: exception})
+                    self.failures.append({node['name']: response})
                 if len(nodes) == 1 and self.summarize_for_single_node:
                     return {vector_name: response}
                 result[node['name']] = {vector_name: response}
@@ -427,7 +440,8 @@ class BottleClient:
         """
         """
         if not self._connected:
-            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
+            raise ConnectionError(
+                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
         if nodes is None:
             nodes = self.all_nodes
         else:
@@ -435,13 +449,13 @@ class BottleClient:
         result = {}
         for node in nodes:
             try:
-                response = requests.post(f"{self.url}{node['id']}/gene/increment-recall-threshold", 
-                                            verify=self.secure, headers=self.headers, 
-                                            json={'increment': increment}).json()
+                response = requests.post(f"{self.url}{node['id']}/gene/increment-recall-threshold",
+                                         verify=self.secure, headers=self.headers,
+                                         json={'increment': increment}).json()
                 if 'error' in response or response['status'] == 'failed':
                     if len(nodes) == 1 and self.summarize_for_single_node:
                         raise Exception("Request Failure:", response)
-                    self.failures.append({node['name']: exception})
+                    self.failures.append({node['name']: response})
                 else:
                     self.genome.primitives[node['id']]['recall_threshold'] += increment
                 if len(nodes) == 1 and self.summarize_for_single_node:
@@ -509,7 +523,8 @@ class BottleClient:
     def ping(self, nodes: List = None) -> List[Dict[str, Any]]:
         """Ping a node to ensure it's up."""
         if not self._connected:
-            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
+            raise ConnectionError(
+                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
         if nodes is None:
             return requests.get(f'{self.url}bottle/ping', headers=self.headers).json()
         else:
@@ -535,7 +550,8 @@ class BottleClient:
         where node-name is the *name* of a primitive or manipulative.
         """
         if not self._connected:
-            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
+            raise ConnectionError(
+                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
         results = {}
         if nodes is None:
             nodes = [node['name'] for node in self.all_nodes]
@@ -547,7 +563,7 @@ class BottleClient:
             if 'error' in response or response['status'] == 'failed':
                 if len(nodes) == 1 and self.summarize_for_single_node:
                     raise Exception("Request Failure:", response)
-                self.failures.append({node['name']: exception})
+                self.failures.append({node['name']: response})
                 print("Failure:", {node['name']: response})
             if len(nodes) == 1 and self.summarize_for_single_node:
                 return response['message']
@@ -562,7 +578,8 @@ class BottleClient:
         where node-name is the *name* of a primitive or manipulative.
         """
         if not self._connected:
-            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
+            raise ConnectionError(
+                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
         results = {}
         if nodes is None:
             nodes = [node['name'] for node in self.all_nodes]
@@ -589,14 +606,16 @@ class BottleClient:
         if nodes is None:
             nodes = self.all_nodes
         return self._query(requests.post, 'kb/analyze', nodes=nodes)
-    
+
     def kb_analysis_progress(self):
         """
         Monitors the KB analysis progress kickstarted by analyze_kb.
         """
         if not self._connected:
-            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
-        response = requests.get(f'{self.url}kb/analysis-progress', verify=self.secure, json=[], headers=self.headers).json()
+            raise ConnectionError(
+                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
+        response = requests.get(f'{self.url}kb/analysis-progress', verify=self.secure, json=[],
+                                headers=self.headers).json()
         return response
 
     def set_target_class(self, target_class: str, nodes: List = None) -> List:
@@ -605,11 +624,9 @@ class BottleClient:
         if nodes is None:
             nodes = self.query_nodes
         return self._query(requests.post, 'set-target-class', nodes=nodes, data=target_class)
-    
+
     def clear_target_class(self, nodes: List = None) -> List:
         "Clears target selection."
         if nodes is None:
             nodes = self.query_nodes
-        return self._query(requests.post, 'clear-target-class', nodes=nodes)
-
-
+        return self._query(requests.post, 'clear-target-class', nodes=nodes)
\ No newline at end of file
